      subroutine readHB(lunit,m,n)
      implicit none
c
c     Read matrix in Harwell-Boeing format
c
      include 'HB.h'
       
      integer lunit,m,n
      CHARACTER      TITLE*72 , KEY*8    , MXTYPE*3 , RHSTYP*3,
     1               PTRFMT*16, INDFMT*16, VALFMT*20, RHSFMT*20

      INTEGER        TOTCRD, PTRCRD, INDCRD, VALCRD, RHSCRD,
     1               NELTVL, I,  NRHS  , NRHSIX
      logical lsame
      external lsame

C    ------------------------
C     ... READ IN HEADER BLOCK
C     ------------------------

      READ ( LUNIT, 1000 ) TITLE , KEY   ,
     1                     TOTCRD, PTRCRD, INDCRD, VALCRD, RHSCRD,
     2                     MXTYPE, NROW  , NCOL  , NNZERO, NELTVL,
     3                     PTRFMT, INDFMT, VALFMT, RHSFMT
      IF  ( RHSCRD .GT. 0 )
     1    READ ( LUNIT, 1001 ) RHSTYP, NRHS, NRHSIX
 1000 FORMAT ( A72, A8 / 5I14 / A3, 11X, 4I14 / 2A16, 2A20 )
 1001 FORMAT ( A3, 11X, 2I14 )

      write (*,1000) TITLE , KEY   ,
     1                     TOTCRD, PTRCRD, INDCRD, VALCRD, RHSCRD,
     2                     MXTYPE, NROW  , NCOL  , NNZERO, NELTVL,
     3                     PTRFMT, INDFMT, VALFMT, RHSFMT
C     -------------------------
C     ... READ MATRIX STRUCTURE
C     -------------------------

      if (ncol.gt.nmax) stop 'ERROR in readHB: ncol > nmax'
      if (nrow.gt.mmax) stop 'ERROR in readHB: nrow > mmax'
      if (nnzero.gt.nnzmax) stop 'ERROR in readHB: nnzero > nnzmax'
      

      READ ( LUNIT, PTRFMT ) ( COLPTR (I), I = 1, NCOL+1 )

      READ ( LUNIT, INDFMT ) ( ROWIND (I), I = 1, NNZERO )

      IF  ( VALCRD .GT. 0 )  THEN

C         ----------------------
C         ... READ MATRIX VALUES
C         ----------------------

          READ ( LUNIT, VALFMT ) ( VALUES (I), I = 1, NNZERO )

      ENDIF

      if (lsame(MXTYPE(2:2),'U').or.lsame(MXTYPE(2:2),'R')) then
         issym = 0
      else if (lsame(MXTYPE(2:2),'S')) then
         issym = 1
      else
         print *, 'unsupported format',MXTYPE
         stop 
      endif

      m = nrow
      n = ncol
      end


      subroutine hbmv(transa,m,n,x,y,dparm,iparm)
      implicit none
      character*1 transa
      integer m,n,iparm(*)
      double precision x(*),y(*),dparm(*)
      logical lsame
      external lsame
      
      if (lsame(transa,'n')) then
         call avHB (m, n, x, y)
      else
         call atvHB (m, n, x, y)
      endif
      end

      subroutine avHB (m, n, x, y)
c
c     computes  w <- A*x
c
      implicit none
      integer          m, n, i, j
      Double precision x(n), y(m)
      include 'HB.h'
      

      do i=1,m
         y(i) = 0d0
      enddo
      if (issym.eq.1) then
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
               y(rowind(j)) = y(rowind(j)) + values(j)*x(rowind(j))
            enddo
         enddo
      else
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(rowind(j)) = y(rowind(j)) + values(j)*x(i)
            enddo
         enddo
      endif
      end

      subroutine atvHB (m, n, x, y)
c
c     computes  y <- A'*x
c     
      integer          m, n, i, j
      Double precision x(n), y(m)
      include 'HB.h'
      
      
      do i=1,n
         y(i) = 0d0
      enddo
      if (issym.eq.1) then
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
               y(rowind(j)) = y(rowind(j)) + values(j)*x(rowind(j))
            enddo
         enddo
      else
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
            enddo
         enddo
      endif
      end




