      program example

      implicit none

      include 'HB.h'

c     %------------%
c     | Parameters |
c     %------------%
      integer lwrk,maxiter,liwrk
      parameter(lwrk=12*KMAX+6 + 8*KMAX**2 + MMAX+NMAX+4*KMAX+4
     c     + MMAX + NMAX + 2*KMAX+2 + MMAX+NMAX+KMAX+1,maxiter=1000,
     c     liwrk = 40*kmax)
      double precision dzero, done
      parameter(dzero = 0.0, done = 1.0)

c     %-----------------%
c     | Local variables |
c     %-----------------%
      character*256 filename
      integer dim,i,m,n,k,ioption(10),iwork(mmax+nmax+10*kmax)
      integer info,iparm(1)
      double precision U(mmax,kmax+2),V(nmax,kmax),work(lwrk)
      double precision doption(10),B(kmax,2),r(mmax),dparm(1)
      double precision tol,bnd(kmax+1)
c     Save large arrays in a common block to avoid stack overflow on
c     certain machines.
      common/cexample/U,V,B,r,work,iwork

c     %--------------------%
c     | External Functions |
c     %--------------------%
      external hbmv

c-------------------- Here begins executable code ---------------------

c      
c     Read filename and number of singular values from stdin.
c
      write (*,*) 'Name of file with matrix in Harwell-Boeing',
     c     ' format?'
      read (5,'(a)') filename
      write(*,*) filename
      write (*,*) 'Desired number of singular values?'
      read (5,*) k
      write (*,*) k
      write (*,*) 'Maximal dimension of subspace?'
      read (5,*) dim
      write (*,*) dim

c
c     Read matrix in Harwell-Boeing format.
c
      write (*,*) 'Reading matrix from file ',filename
      open(10,file=filename,status='old')
      call readHB(10,m,n)
      close(10)
      write (*,*) '...done'
      write (*,*) 'Dimensions of matrix: m,n = ',m,n
      if (m.gt.mmax .or. n.gt.nmax) then
         write (*,'(a, I6, a, i6)') 'Matrix kan be at most',mmax,' by ',
     c        nmax
         stop 'Matrix dimensions exceed allocated size'
      endif
         
c
c     Set the first column of U to zero. This will make dlansvd_irl
c     generate a random starting vector.      
c
      do i=1,m
         U(i,1) = 1d0
      enddo

c
c     Clear statistics counters and timers
c
      call clearstat

c
c     Set options for DLANSVD_IRL
c
      ioption(1) = 0
      ioption(2) = 1      
      doption(1) = 1d-8
      doption(2) = 1d-12
      doption(3) = dzero
      doption(4) = 0.005
      tol = 16d-16
c
c     Compute the k largest singular values
c
      write (*,*) 'Computing the ',k,' largest singular values.'
      dim = min(dim,kmax)
      call dlansvd_irl('n','n',m,n,k,dim,maxiter,hbmv,U,mmax,B,bnd,
     c     V,nmax,tol,work,lwrk,iwork,liwrk,doption,ioption,info,
     c     dparm,iparm)

c
c     Print singular values and computed error bounds.
c
      write(*,*) '  i          sigma(i)              bnd(i)'
      do i=1,k
         write(*,'(1I5,1p,1e25.15,1p,1e12.3)') i,B(i,1),bnd(i)
      enddo 

c
c     Print runtime statistics 
c
      call printstat
      end

