      subroutine dritzvec(jobu,jobv,m,n,k,dim,D,E,S,U,ldu,V,ldv,
     c     work,in_lwrk,iwork)

c
c     Compute Ritz-vectors.
c

c     %----------------------------------------------------------------------%
c     | The bidiagonal SVD is computed in a two-stage procedure:             
c     |                                                   
c     | 1. Compute a QR-factorization M^T*B = [R; 0] of the (k+1)-by-k lower 
c     |    bidiagonal matrix B.
c     | 2. Compute the SVD of the k-by-k upper bidiagonal matrix 
c     |    R = P*S*Q^T. The SVD of B is then (M*P)*S*Q^T.
c     %----------------------------------------------------------------------%
      
c     %-----------%
c     | Arguments |
c     %-----------%
      implicit none
      include 'stat.h'
      character*1 jobu,jobv
      integer m,n,k,dim,ldu,ldv,in_lwrk,iwork(*)
      double precision U(ldu,*),V(ldv,*),D(*),E(*),S(*),work(*)

c     %------------%
c     | Parameters |
c     %------------%
      double precision one, zero
      parameter(one = 1.0, zero = 0.0)

c     %-----------------%
c     | Local variables |
c     %-----------------%
      integer lwrk
      integer ip,iqt,imt,iwrk,id,info
      double precision c1,c2,dd
      real t0,t1

c     %--------------------%
c     | External Functions |
c     %--------------------%
      logical lsame
      external lsame, dbdqr, dbdsdc, dgemm_ovwr_left
      
c-------------------- Here begins executable code ---------------------

      call second(t0)
      iwrk = 1
      lwrk = in_lwrk
      
c     %-----------------------------------------%
c     | Compute QR-factorization
c     |   B = M * [R; 0]
c     %-----------------------------------------%
      imt = 1
      iqt = imt + (dim+1)**2 
      ip = iqt + dim**2
      iwrk = ip + dim**2
      lwrk = lwrk - iwrk + 1      
      call dbdqr(jobu,dim,D,E,c1,c2,work(imt),dim+1)
      
c     %-----------------------------------------%
c     | Compute SVD of R:
c     |      R = P * S * Q^T, 
c     | using the Divide-and-conquer SVD
c     %-----------------------------------------%
      call dbdsdc('u','I',dim,D,E,work(ip),dim,work(iqt),dim,dd,id,
     c     work(iwrk),iwork,info)
      
c     %-----------------------------------------%
c     | Compute left singular vectors for B
c     |    X = P^T * M^T 
c     %-----------------------------------------%
      call dgemm_ovwr('t',dim,dim+1,dim,1d0,work(ip),dim,0d0,
     c     work(imt),dim+1,work(iwrk),lwrk)
      
      
      if (lsame(jobu,'y')) then
c     %-----------------------------------------%
c     | Form left Ritz-vectors                  |
c     |   U = U * X^T                           |
c     %-----------------------------------------%
         call dgemm_ovwr_left('t',m,k,dim+1,1d0,U,ldu,0d0,
     c        work(imt),dim+1,work(iwrk),lwrk)
      endif
      
      if (lsame(jobv,'y')) then
c     %-----------------------------------------%
c     | Form right Ritz-vectors
c     |   V = V * Q
c     %-----------------------------------------%
         call dgemm_ovwr_left('t',n,k,dim,1d0,V,ldv,0d0,
     c        work(iqt),dim,work(iwrk),lwrk)
      endif    
      
      call  second(t1)           
      tritzvec = t1-t0
      end
      
